# Generated by Django 4.2.20 on 2025-06-02 21:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("celus_registry", "0001_initial")]

    operations = [
        migrations.CreateModel(
            name="Notification",
            fields=[
                ("id", models.UUIDField(primary_key=True, serialize=False)),
                ("source", models.CharField(max_length=16)),
                ("type", models.CharField(max_length=12)),
                ("start_date", models.DateField(blank=True, null=True)),
                ("end_date", models.DateField(blank=True, null=True)),
                ("subject", models.CharField(max_length=128)),
                ("message", models.TextField(blank=True)),
                (
                    "published_date",
                    models.DateTimeField(
                        blank=True,
                        help_text="When the notification was published",
                        null=True,
                    ),
                ),
                ("last_modified", models.DateTimeField()),
                (
                    "sushi_service",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="notifications",
                        to="celus_registry.sushiservice",
                    ),
                ),
            ],
        )
    ]
