from django.apps import AppConfig


class CelusRegistryConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_celus_registry"
    verbose_name = "Celus Registry"
    label = "celus_registry"
